use async_trait::async_trait;
use mumble_protocol::control::ControlPacket;
use mumble_protocol::voice::{Clientbound, Serverbound, VoicePacket, VoicePacketDst};
use std::io::Error;

pub trait Message<Dst: VoicePacketDst>: Into<ControlPacket<Dst>> + Send {
    fn serialize(self) -> Vec<u8>;
}

#[async_trait]
pub trait ControlChannel: Send + Sync {
    async fn send(&self, message: ControlPacket<Clientbound>) -> Result<(), Error>;

    async fn receive(&self) -> Result<ControlPacket<Serverbound>, Error>;

    fn get_stats(&self) -> ControlChannelStats;
}

#[async_trait]
pub trait AudioChannel: Send + Sync {
    async fn send(&self, packet: VoicePacket<Clientbound>) -> Result<(), Error>;

    async fn receive(&self) -> Result<VoicePacket<Serverbound>, Error>;

    fn get_stats(&self) -> AudioChannelStats;
}

pub struct ControlChannelStats {
    pub received: u32,
}

pub struct AudioChannelStats {
    pub good: u32,
    pub late: u32,
    pub lost: u32,
    pub received: u32,
}
